@@ -492,6 +492,75 @@ garcon_gtk_menu_reload (GarconGtkMenu *menu)
 
 
 
+static GtkWidget*
+garcon_gtk_menu_load_icon (const gchar *icon_name)
+{
+  GtkWidget *image = NULL;
+  gint w, h, size;
+  gchar *p, *name = NULL;
+  GdkPixbuf *pixbuf = NULL;
+  GtkIconTheme *icon_theme = gtk_icon_theme_get_default ();
+
+  gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
+  size = MIN (w, h);
+
+  if (gtk_icon_theme_has_icon (icon_theme, icon_name))
+    {
+      image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
+    }
+  else
+    {
+      if (g_path_is_absolute (icon_name))
+        {
+          pixbuf = gdk_pixbuf_new_from_file_at_scale (icon_name, w, h, TRUE, NULL);
+        }
+      else
+        {
+          /* try to lookup names like application.png in the theme */
+          p = strrchr (icon_name, '.');
+          if (p)
+            {
+              name = g_strndup (icon_name, p - icon_name);
+              pixbuf = gtk_icon_theme_load_icon (icon_theme, name, size, 0, NULL);
+              g_free (name);
+              name = NULL;
+            }
+
+          /* maybe they point to a file in the pixbufs folder */
+          if (G_UNLIKELY (pixbuf == NULL))
+            {
+              gchar *filename;
+
+              filename = g_build_filename ("pixmaps", icon_name, NULL);
+              name = xfce_resource_lookup (XFCE_RESOURCE_DATA, filename);
+              g_free (filename);
+            }
+
+          if (name)
+            {
+              pixbuf = gdk_pixbuf_new_from_file_at_scale (name, w, h, TRUE, NULL);
+              g_free (name);
+            }
+        }
+
+      /* Turn the pixbuf into a gtk_image */
+      if (G_LIKELY (pixbuf))
+        {
+          /* scale the pixbuf down if it needs it */
+          GdkPixbuf *tmp = gdk_pixbuf_scale_simple (pixbuf, w, h, GDK_INTERP_BILINEAR);
+          g_object_unref (pixbuf);
+          pixbuf = tmp;
+
+          image = gtk_image_new_from_pixbuf (pixbuf);
+          g_object_unref (G_OBJECT (pixbuf));
+        }
+    }
+
+  return image;
+}
+
+
+
 static gboolean
 garcon_gtk_menu_add (GarconGtkMenu *menu,
                      GtkMenu       *gtk_menu,
@@ -517,6 +586,8 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
 
       if (GARCON_IS_MENU_ITEM (li->data))
         {
+          GtkWidget *box, *label;
+
           /* watch for changes */
           g_signal_connect_swapped (G_OBJECT (li->data), "changed",
               G_CALLBACK (garcon_gtk_menu_reload), menu);
@@ -536,7 +607,36 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
             continue;
 
           /* create item */
-          mi = gtk_image_menu_item_new_with_label (name);
+          mi = gtk_menu_item_new ();
+          label = gtk_label_new (name);
+          gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+#if GTK_CHECK_VERSION (3, 0, 0)
+          box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
+          box = gtk_hbox_new (FALSE, 0);
+          gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5f);
+#endif
+
+          if (menu->priv->show_menu_icons)
+            {
+              icon_name = garcon_menu_item_get_icon_name (li->data);
+              if (STR_IS_EMPTY (icon_name))
+                icon_name = "applications-other";
+
+              image = garcon_gtk_menu_load_icon (icon_name);
+              gtk_widget_show (image);
+            }
+          else
+            {
+              image = gtk_image_new ();
+            }
+
+          /* Add the image and label to the box, add the box to the menu item */
+          gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
+          gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 6);
+          gtk_widget_show_all (box);
+          gtk_container_add (GTK_CONTAINER (mi), box);
+
           gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
           g_signal_connect (G_OBJECT (mi), "activate",
               G_CALLBACK (garcon_gtk_menu_item_activate), li->data);
@@ -564,17 +664,6 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
           if (STR_IS_EMPTY (command))
             gtk_widget_set_sensitive (mi, FALSE);
 
-          if (menu->priv->show_menu_icons)
-            {
-              icon_name = garcon_menu_item_get_icon_name (li->data);
-              if (STR_IS_EMPTY (icon_name))
-                icon_name = "applications-other";
-
-              image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
-              gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
-              gtk_widget_show (image);
-            }
-
           /* atleast 1 visible child */
           has_children = TRUE;
         }
@@ -596,16 +685,22 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
             continue;
 
           submenu = gtk_menu_new ();
+          gtk_menu_set_reserve_toggle_size (GTK_MENU (submenu), FALSE);
           if (garcon_gtk_menu_add (menu, GTK_MENU (submenu), li->data))
             {
+              GtkWidget *box, *label;
+
               /* attach submenu */
               name = garcon_menu_element_get_name (li->data);
-              mi = gtk_image_menu_item_new_with_label (name);
-              gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
-              gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), submenu);
-              g_signal_connect (G_OBJECT (submenu), "selection-done",
-                  G_CALLBACK (garcon_gtk_menu_deactivate), menu);
-              gtk_widget_show (mi);
+              mi = gtk_menu_item_new ();
+              label = gtk_label_new (name);
+              gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+#if GTK_CHECK_VERSION (3, 0, 0)
+              box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+#else
+              box = gtk_hbox_new (FALSE, 0);
+              gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5f);
+#endif
 
               if (menu->priv->show_menu_icons)
                 {
@@ -614,9 +709,24 @@ garcon_gtk_menu_add (GarconGtkMenu *menu,
                     icon_name = "applications-other";
 
                   image = gtk_image_new_from_icon_name (icon_name, GTK_ICON_SIZE_MENU);
-                  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mi), image);
                   gtk_widget_show (image);
                 }
+              else
+                {
+                  image = gtk_image_new ();
+                }
+
+              /* Add the image and label to the box, add the box to the menu item */
+              gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);
+              gtk_box_pack_start (GTK_BOX (box), label, TRUE, TRUE, 6);
+              gtk_widget_show_all (box);
+              gtk_container_add (GTK_CONTAINER (mi), box);
+
+              gtk_menu_shell_append (GTK_MENU_SHELL (gtk_menu), mi);
+              gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), submenu);
+              g_signal_connect (G_OBJECT (submenu), "selection-done",
+                  G_CALLBACK (garcon_gtk_menu_deactivate), menu);
+              gtk_widget_show (mi);
             }
           else
             {
